"""
Provides Nose and Django test case assert functions
"""
from contextlib import contextmanager

from nose.tools import (
    assert_equal, assert_true, assert_false, assert_in, assert_not_in, assert_raises, assert_not_equal, assert_is,
    assert_is_instance, assert_greater, assert_less, assert_almost_equal, assert_not_almost_equal, assert_greater_equal,
    assert_less_equal, assert_not_is_instance, assert_list_equal, assert_tuple_equal, assert_set_equal,
    assert_dict_equal, assert_sequence_equal, assert_multi_line_equal, assert_is_none, assert_is_not_none,
    assert_equals, assert_logs
)

from germanium import config


if config.TURN_OFF_MAX_DIFF:
    assert_equal.__self__.maxDiff = None


def fail(msg=None):
    raise AssertionError(msg)


@contextmanager
def assert_not_raises(exc_type, func=None, *args, **kwargs):
    try:
        if func:
            func(*args, **kwargs)
        yield None
    except exc_type:
        raise fail('{} raised'.format(exc_type.__name__))


def assert_length_equal(iterable, expected_length, msg=None):
    assert_equal(len(iterable), expected_length, msg)


class AllEqual:

    def __eq__(self, obj):
        return True


class NotNoneEqual:

    def __eq__(self, obj):
        return obj is not None


all_eq_obj = AllEqual()
not_none_eq_obj = NotNoneEqual()



import logging

from django.test.client import Client
from django.test.client import RequestFactory

from germanium import config
from germanium.tools import assert_http_redirect, capture_commit_callbacks

from .auth import AuthTestCaseMixin
from .default import GermaniumSimpleTestCase, GermaniumTestCase


class ClientTestCaseMixin(AuthTestCaseMixin):

    logger = logging.getLogger('tests')

    def setUp(self):
        self.c = Client()
        self.r_factory = RequestFactory()
        self.default_headers = {}
        super().setUp()

    def get_request_with_user(self, request):
        request.user = self.logged_user.user
        return request

    def logout(self):
        self.get(config.LOGOUT_URL)
        self.logged_user = None

    def authorize(self, username, password):
        assert_http_redirect(self.post(config.LOGIN_URL, {config.USERNAME: username, config.PASSWORD: password}))

    def get(self, url, headers=None, execute_pre_commit=True, execute_on_commit=False,
            execute_on_commit_cascade=False):
        headers = headers or {}
        headers.update(self.default_headers)

        with capture_commit_callbacks(execute_pre_commit=execute_pre_commit, execute_on_commit=execute_on_commit,
                                      execute_on_commit_cascade=execute_on_commit_cascade):
            resp = self.c.get(url, **headers)
        return resp

    def put(self, url, data={}, headers=None, execute_pre_commit=True, execute_on_commit=False,
            execute_on_commit_cascade=False):
        headers = headers or {}
        headers.update(self.default_headers)

        if execute_on_commit:
            with capture_commit_callbacks(execute_pre_commit=execute_pre_commit, execute_on_commit=execute_on_commit,
                                          execute_on_commit_cascade=execute_on_commit_cascade):
                resp = self.c.put(url, data, **self.headers)
        else:
            resp = self.c.put(url, data, **self.headers)
        return resp

    def post(self, url, data, headers=None, execute_pre_commit=True, execute_on_commit=False,
             execute_on_commit_cascade=False):
        headers = headers or {}
        headers.update(self.default_headers)
        with capture_commit_callbacks(execute_pre_commit=execute_pre_commit, execute_on_commit=execute_on_commit,
                                      execute_on_commit_cascade=execute_on_commit_cascade):
            resp = self.c.post(url, data, **headers)
        return resp

    def head(self, url, headers=None, execute_pre_commit=True, execute_on_commit=False,
             execute_on_commit_cascade=False):
        headers = headers or {}
        headers.update(self.default_headers)

        with capture_commit_callbacks(execute_pre_commit=execute_pre_commit, execute_on_commit=execute_on_commit,
                                      execute_on_commit_cascade=execute_on_commit_cascade):
            resp = self.c.head(url, **headers)
        return resp

    def options(self, url, headers=None, execute_pre_commit=True, execute_on_commit=False,
                execute_on_commit_cascade=False):
        headers = headers or {}
        headers.update(self.default_headers)

        with capture_commit_callbacks(execute_pre_commit=execute_pre_commit, execute_on_commit=execute_on_commit,
                                      execute_on_commit_cascade=execute_on_commit_cascade):
            resp = self.c.options(url, **headers)
        return resp

    def delete(self, url, headers=None, execute_pre_commit=True, execute_on_commit=False,
               execute_on_commit_cascade=False):
        headers = headers or {}
        headers.update(self.default_headers)

        with capture_commit_callbacks(execute_pre_commit=execute_pre_commit, execute_on_commit=execute_on_commit,
                                      execute_on_commit_cascade=execute_on_commit_cascade):
            resp = self.c.delete(url, **headers)
        return resp


class ClientTestCase(ClientTestCaseMixin, GermaniumTestCase):
    pass


class SimpleClientTestCase(ClientTestCaseMixin, GermaniumSimpleTestCase):
    pass
